/*----------------------------------------------------------------------------
| File:
|   CANGEN.C
| Project:
|   Example application using VCAND.DLL to generate cyclic CAN messages
|
|-----------------------------------------------------------------------------
| $Author: Bedrich $    $Locker: $   $Revision: 30 $
| $Header: /CANdriver/Samples/CANgen/cangen.c 30    30.09.03 7:44 Bedrich $
|-----------------------------------------------------------------------------
| Copyright (c) 2000 by Vector Informatik GmbH.  All rights reserved.
 ----------------------------------------------------------------------------*/
 
#define  STRICT
#include <windows.h>

#include "stdio.h"
#include "conio.h"
#include "string.h"
#include "dos.h"

#include "..\dll\VCanD.h"


/////////////////////////////////////////////////////////////////////////
// Globals

VportHandle  gPortHandle     = INVALID_PORTHANDLE;
Vaccess      gChannelMask    = 0;
Vaccess      gPermissionMask = 0;
unsigned int gCanId          = 100;
unsigned int gBitRate        = 1000000;
unsigned int gSendCycle      = 1000; // ms
unsigned int gSendBurst      = 1;
int          gLogEvents      = 0;
unsigned int gHwType         = HWTYPE_CANCARDX;
int          gHwChannel      = 1;
HANDLE       gEventHandle    = 0;


////////////////////////////////////////////////////////////////////////
// InitDriver ()
//---------------------------------------------------------------------
// Initializes the CAN driver interface

static Vstatus InitDriver(void)
{
  Vstatus vErr;
  VsetAcceptance acc;

  // Open the driver
  vErr = ncdOpenDriver ();
  if (vErr) goto error;

  // Select the channel
  gChannelMask = ncdGetChannelMask(gHwType,0,gHwChannel);
  if (!gChannelMask) {
    printf("ERROR: Channel not present!\n");
    vErr = VERROR;
    goto error;
  }

  // Open a port
  vErr = ncdOpenPort(&gPortHandle,"CANgen",gChannelMask,gChannelMask,&gPermissionMask,128);
  if (vErr) goto error;

  // Only if permission to initialize the channel
  if (gPermissionMask) {
    // Set the bus timing
    vErr = ncdSetChannelBitrate(gPortHandle,gPermissionMask,gBitRate);
    if (vErr) goto error;
  } else if (gBitRate) {

    printf("WARNING: No init access! Bitrate ignored.\n");

  }

  // Disable the TX and TXRQ notifications
  vErr = ncdSetChannelMode(gPortHandle,gChannelMask,1,0);
  if (vErr) goto error;

  // Close the the acceptance filter
  acc.mask = 0x7FF; // relevant=1
  acc.code = 0;
  vErr = ncdSetChannelAcceptance(gPortHandle,gChannelMask,&acc);
  if (vErr) goto error;

  return VSUCCESS;

error:

  printf("ERROR: %s!\n",ncdGetErrorString(vErr));

  if (gPortHandle!=INVALID_PORTHANDLE) {
    ncdClosePort(gPortHandle);
    gPortHandle = INVALID_PORTHANDLE;
  }

  return vErr;
}                                                  // end InitDriver ()



///////////////////////////////////////////////////////////////////////
// CleanUp()
//---------------------------------------------------------------------
// Close the CAN driver interface

static Vstatus CleanUp(void)
{
  ncdClosePort(gPortHandle);
  gPortHandle = INVALID_PORTHANDLE;
  ncdCloseDriver();
  return VSUCCESS; // No error handling
}


///////////////////////////////////////////////////////////////////////
// main()
//---------------------------------------------------------------------
//

// help
static void usage( void ) {

  printf(
    "Usage:\n"
    "  CANgen [options]\n"
    "\n"
    "  Options:\n"
    "    -ix         Set identifier to x            (default=100)\n"
    "    -bx         Set bitrate to x               (default=1000 kBaud)\n"
    "    -cx         Set transmission cycle to x ms (default=1000)\n"
    "    -chx        Set channel                    (default=2)\n"
    "    -nx         Set burst size to x msgs       (default=1)\n"
    "    -v          Use virtual channel 1          (default=CANcardX channel 2)\n"
    "    -p          Use CANpari\n"
    "    -a          Use CAN-AC2\n"
    "    -pci        Use CAN-AC2-PCI\n"
    "    -CANcardXL  Use CANcardXL\n"
    "    -CANcaseXL  Use CANcaseXL\n"
    "    -CANboardXL use CANboardXL\n"
    "    -l          Log events to screen\n"
    "    -ESC        EXIT\n"
    );
  exit(1);
}

HANDLE threadHandle = 0;
DWORD threadId = 0;
int threadRuning = 0;


DWORD WINAPI thread( PVOID par ) {

  Vstatus       vErr = 0;
  Vevent        event;
  Vevent       *pEvent;
  unsigned int  actTime,nextSendTime;
  unsigned int  i;

  event.tag                 = V_TRANSMIT_MSG;
  event.tagData.msg.id      = gCanId;
  event.tagData.msg.flags   = 0;
  event.tagData.msg.dlc     = 8;
  event.tagData.msg.data[0] = 0;
  event.tagData.msg.data[1] = 0;
  event.tagData.msg.data[2] = 0;
  event.tagData.msg.data[3] = 0;
  event.tagData.msg.data[4] = 0;
  event.tagData.msg.data[5] = 0;
  event.tagData.msg.data[6] = 0;
  event.tagData.msg.data[7] = 0;

  actTime      = 0;
  nextSendTime = 0;
  threadRuning = 1;
  while (threadRuning) {

    // wait for receive event
    WaitForSingleObject(gEventHandle,1000);

    // Poll for received events
    for (;;) {

      vErr = ncdReceive1(gPortHandle,&pEvent);
      if (vErr==VSUCCESS) { // Ok

        if (gLogEvents) printf("%s\n",ncdGetEventString(pEvent));

        if (pEvent->timeStamp<actTime) { // Time has been reset
          if (gLogEvents) printf("WARNING: Time has been reset, time = %u!\n",pEvent->timeStamp);
          nextSendTime = 0;
        }

        actTime = pEvent->timeStamp;

        // Send
        if (nextSendTime<=actTime) {
          static unsigned long Count = 0;
          Count++;
          *((unsigned long *) &event.tagData.msg.data[0]) = Count;
          for (i=0;i<gSendBurst;i++) {
            if (gCanId==0) {
              event.tagData.msg.id++;
              if (event.tagData.msg.id>2047) event.tagData.msg.id = 0;
            }
            vErr =  ncdTransmit(gPortHandle, gChannelMask, &event);
            if (vErr==VERR_QUEUE_IS_FULL) {
              printf("!");
              break;
            }
            else if (vErr) goto error;
            event.tagData.msg.data[1] = i;
          }

          // actTime may not start at zero
          // because ncdResetClock needs initialize permisson
          if (nextSendTime==0) nextSendTime = actTime;
          nextSendTime = nextSendTime+100*gSendCycle;

        }

      }
      else if (vErr!=VERR_QUEUE_IS_EMPTY) { // Error
        goto error;
      }
      else { // Queue empty
        vErr = VSUCCESS;
        break;
      }
    }



  }

error:
  if (vErr) printf("ERROR: %s!\n",ncdGetErrorString(vErr));
  return 0;
}


int main( int argc, char *argv[])
{
  Vstatus       vErr;
  int           i;
  int           end = 0;

  printf("CANgen message generator example application\n");
  printf("1998, Vector Informatik GmbH\n\n");

  // Commandline
  for (i=1; i<argc; i++) {
    unsigned int b;
    if (strcmp(argv[i], "-h") == 0) {
        usage();
    } else if (strcmp(argv[i], "-v") == 0) {
        gHwType = HWTYPE_VIRTUAL;
    } else if (strcmp(argv[i], "-p") == 0) {
        gHwType = HWTYPE_CANPARI;
        gHwChannel = 0;
    } else if (strcmp(argv[i], "-a") == 0) {
        gHwType = HWTYPE_CANAC2;
    } else if (strcmp(argv[i], "-pci") == 0) {
        gHwType = HWTYPE_CANAC2PCI;
    } else if (stricmp(argv[i], "-CANcardXL") == 0) {
        gHwType = HWTYPE_CANCARDXL;
    } else if (stricmp(argv[i], "-CANcaseXL") == 0) {
        gHwType = HWTYPE_CANCASEXL;
    } else if (stricmp(argv[i], "-CANboardXL") == 0) {
        gHwType = HWTYPE_CANBOARDXL;
    } else if (strcmp(argv[i], "-l") == 0) {
        gLogEvents++;
    } else if (sscanf(argv[i], "-b%u", &b) == 1) {
        gBitRate = b;
    } else if (sscanf(argv[i], "-i%u", &b) == 1) {
        gCanId = b;
    } else if (sscanf(argv[i], "-c%u", &b) == 1) {
        gSendCycle = b;
    } else if (sscanf(argv[i], "-ch%u", &b) == 1) {
        gHwChannel = b-1;
    } else if (sscanf(argv[i], "-n%u", &b) == 1) {
        gSendBurst = b;
    } else {
        usage();
    }
  }
  if(( gHwType == HWTYPE_CANPARI) && (gHwChannel == 1)){
    gHwChannel = 0;
    printf("Warning: Only channel 1 available with CANpari!\n");
  }
  printf("bitrate = %u\n",gBitRate);
  printf("id      = %u\n",gCanId);
  printf("channel = %u \n",gHwChannel+1);
  printf("cycle   = %u ms\n",gSendCycle);
  printf("burst   = %u\n",gSendBurst);

  // Initialize the CAN driver
  vErr = InitDriver();
  if (vErr) goto finish;

  // create a synchronisation object
  gEventHandle = CreateEvent(NULL, FALSE, FALSE, NULL);
  vErr = ncdSetNotification(gPortHandle, (unsigned long*)&gEventHandle, 1);
  if (vErr) goto error;


  // Go on bus
  vErr = ncdActivateChannel(gPortHandle,gChannelMask);
  if (vErr) goto error;

  // Setup cyclic timer messages
  vErr = ncdSetTimerRate(gPortHandle,100*gSendCycle);
  if (vErr) goto error;

  // create a thread
  threadHandle = CreateThread(0,0x1000,thread,0,0,&threadId);
  SetThreadPriority(threadHandle,THREAD_PRIORITY_TIME_CRITICAL);

  while (!end) {

    Sleep(500);

    // Keyboard
    if (kbhit()) {
      switch (getch()) {
        case 27:
          goto finish;
        case 'h':
        case '+':
          gSendBurst++;
          printf("burst   = %u\n",gSendBurst);
          break;
        case 'g':
        case '-':
          if (gSendBurst>1) gSendBurst--;
          printf("burst   = %u\n",gSendBurst);
          break;
      }
    }
  }

error:

  if (vErr) printf("ERROR: %s!\n",ncdGetErrorString(vErr));

finish:

  // stop thread
  if (threadHandle) {
    threadRuning = 0;
    WaitForSingleObject(threadHandle,5000);
  }

  ncdDeactivateChannel(gPortHandle, gChannelMask);
  CleanUp();

  return 0;
}

